/*
 *  GOL - Game of life simulation software
 *  Copyright (C) 1996 Michael Raymond Brown
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Author: Michael Raymond Brown
 *  EMail:  michael@arrakeen.demon.co.uk
 *  WWW:    http://www.arrakeen.demon.co.uk/
 *  Address:  Michael Brown
 *            Kia Ora
 *            Brook Street
 *            Great Bromley
 *            Nr Colchester
 *            Essex
 *            CO7 7HX
 *            England
 */


#ifndef _Life_h
#define _Life_h

#ifdef __GNUC__
#pragma interface
#endif


#include "Image.h"

class Life
   {public:
	char rules[22];

	enum {CLEAR = 1 };
	enum {LEAVE = 2 };
	enum {SET   = 3 };

	Life();
	virtual ~Life();

	  /**** Do some processing
	   * return: 0 if more calls to tick are needed
	   *	     1 if the life field is ready for display
	   */
	virtual int tick() = 0;

	  /**** Check to see if the life field
	   * is ready for display.
	   */
	virtual int isOk() = 0;

	  /**** Set a pixel in the life field
	   */
	virtual void plot(int x, int y, int value) = 0;

	  /**** Get a pixel from the life field
	   */
	virtual int point(int x, int y) = 0;

	  /**** Get life field dimensions
	   */
	virtual int getWidth() = 0;
	virtual int getHeight() = 0;

	virtual void getArea(
		     int x, int y // Co-ordinates of a 16x16 square
		   , u_int* dst) = 0;
	virtual void setArea(
		     int x, int y // Co-ordinates of a 16x16 square
		   , const u_int* src) = 0;

	virtual void clear() = 0;
	virtual void randomize() = 0;
   };

class local_Life : public Life
   {private:
	typedef unsigned int u_int;
	int	 width;
	int	height;

	int  scan_width;
	int scan_height;

	int	  wrap;

	int    reverse;	// True when 'field' points 1 scan line ahead
	u_int*	 field;
	u_int*	 spare;	// Spare scan line at the end of the buffer
			// if wrap==True, Copy of first scan line
			// if wrap==False, Zeros

	u_int *p1, *p2, *p3, *dst;
	int   scan_count;
	int linespertick;

	u_char	lookup[65536];

	inline void tick_scan(u_int v1, u_int v2, u_int v3, const u_int* p1, const u_int* p2, const u_int* p3, u_int* dst);
	int tick_up();
	int tick_down();
	int tick_upw();
	int tick_downw();

	int (local_Life::*tickf)();

    public:
	  /**** Create a game of life object
	   * 'width/height' will be rounded up to the nearest multiple of 64
	   * if 'wrap' is true, the life field edges will wrap around.
	   * rules = An array of CLEAR/LEAVE/SET rules, 1 for each pixel count.
	   * mask  = Mask of pixels to include in the count, mask[1][1] is
	   *	     the central pixel. 1=allow, and 0=disallow.
	   *	     mask[0][0] is the upper left hand corner.
	   */
	local_Life(int width, int height, int wrap
		 , char rules[10], char mask[3][3]);
	~local_Life();

	int tick();
	int isOk();
	void plot(int x, int y, int value);
	int point(int x, int y);
	int getWidth();
	int getHeight();

	void getArea(int x, int y	// Co-ordinates of a 16x16 square
		   , u_int* dst);

	void setArea(int x, int y	// Co-ordinates of a 16x16 square
		   , const u_int* src);

	void clear();
	void randomize();
   };

#endif

